Class: Isolate::Sandbox
- Inherits:
-
Object
- Object
- Isolate::Sandbox
- Includes:
- Events
- Defined in:
- lib/isolate/sandbox.rb
Overview
An isolated environment. This class exposes lifecycle events for extension, see Isolate::Events for more information.
Constant Summary collapse
- DEFAULT_PATH =
:nodoc:
"tmp/isolate"
Instance Attribute Summary collapse
-
#entries ⇒ Object
readonly
:nodoc:.
-
#environments ⇒ Object
readonly
:nodoc:.
-
#files ⇒ Object
readonly
:nodoc:.
Instance Method Summary collapse
-
#activate(environment = nil) ⇒ Object
Activate this set of isolated entries, respecting an optional
environment. -
#cleanup ⇒ Object
:nodoc:.
- #cleanup? ⇒ Boolean
- #disable(&block) ⇒ Object
-
#enable ⇒ Object
:nodoc:.
- #enabled? ⇒ Boolean
-
#environment(*environments, &block) ⇒ Object
(also: #env)
Restricts
gemcalls insideblockto a set ofenvironments. -
#gem(name, *requirements) ⇒ Object
Express a gem dependency.
-
#index ⇒ Object
A source index representing only isolated gems.
-
#initialize(options = {}, &block) ⇒ Sandbox
constructor
Create a new Isolate::Sandbox instance.
-
#install(environment) ⇒ Object
:nodoc:.
-
#install? ⇒ Boolean
:nodoc:.
-
#load(file) ⇒ Object
:nodoc:.
-
#log(s) ⇒ Object
:nodoc:.
- #multiruby? ⇒ Boolean
- #options(options = nil) ⇒ Object
- #path ⇒ Object
- #remove(*extra) ⇒ Object
- #system? ⇒ Boolean
- #verbose? ⇒ Boolean
Methods included from Events
Constructor Details
#initialize(options = {}, &block) ⇒ Sandbox
Create a new Isolate::Sandbox instance. See Isolate.now! for the most common use of the API. You probably don’t want to use this constructor directly. Fires :initializing and :initialized.
39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 |
# File 'lib/isolate/sandbox.rb', line 39 def initialize = {}, &block @enabled = false @entries = [] @environments = [] @files = [] @options = fire :initializing user = File. "~/.isolate/user.rb" load user if File.exist? user file, local = nil unless FalseClass === [:file] file = [:file] || Dir["{Isolate,config/isolate.rb}"].first local = "#{file}.local" if file end load file if file if block_given? /\@(.+?):\d+/ =~ block.to_s files << ($1 || "inline block") instance_eval(&block) end load local if local && File.exist?(local) fire :initialized end |
Instance Attribute Details
#entries ⇒ Object (readonly)
:nodoc:
30 31 32 |
# File 'lib/isolate/sandbox.rb', line 30 def entries @entries end |
#environments ⇒ Object (readonly)
:nodoc:
31 32 33 |
# File 'lib/isolate/sandbox.rb', line 31 def environments @environments end |
#files ⇒ Object (readonly)
:nodoc:
32 33 34 |
# File 'lib/isolate/sandbox.rb', line 32 def files @files end |
Instance Method Details
#activate(environment = nil) ⇒ Object
Activate this set of isolated entries, respecting an optional environment. Points RubyGems to a separate repository, messes with paths, auto-installs gems (if necessary), activates everything, and removes any superfluous gem (again, if necessary). If environment isn’t specified, ISOLATE_ENV, RAILS_ENV, and RACK_ENV are checked before falling back to "development". Fires :activating and :activated.
79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 |
# File 'lib/isolate/sandbox.rb', line 79 def activate environment = nil enable unless enabled? fire :activating env = (environment || Isolate.env).to_s install env if install? entries.each do |e| e.activate if e.matches? env end cleanup if cleanup? fire :activated self end |
#cleanup ⇒ Object
:nodoc:
97 98 99 100 101 102 103 104 105 106 107 108 109 |
# File 'lib/isolate/sandbox.rb', line 97 def cleanup # :nodoc: fire :cleaning gem_dir = Gem.dir global, local = Gem::Specification.partition { |s| s.base_dir != gem_dir } legit = legitimize! extra = (local - legit) + (local & global) self.remove(*extra) fire :cleaned end |
#cleanup? ⇒ Boolean
111 112 113 |
# File 'lib/isolate/sandbox.rb', line 111 def cleanup? install? and @options.fetch(:cleanup, true) end |
#disable(&block) ⇒ Object
115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 |
# File 'lib/isolate/sandbox.rb', line 115 def disable &block return self if not enabled? fire :disabling ENV.replace @old_env $LOAD_PATH.replace @old_load_path @enabled = false Isolate.refresh fire :disabled begin; return yield ensure enable end if block_given? self end |
#enable ⇒ Object
:nodoc:
132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 |
# File 'lib/isolate/sandbox.rb', line 132 def enable # :nodoc: return self if enabled? fire :enabling @old_env = ENV.to_hash @old_load_path = $LOAD_PATH.dup path = self.path FileUtils.mkdir_p path ENV["GEM_HOME"] = path unless system? isolate_lib = File. "../..", __FILE__ # manually deactivate pre-isolate gems... is this just for 1.9.1? $LOAD_PATH.reject! do |p| p != isolate_lib && Gem.path.any? { |gp| p.include?(gp) } end # HACK: Gotta keep isolate explicitly in the LOAD_PATH in # subshells, and the only way I can think of to do that is by # abusing RUBYOPT. unless ENV["RUBYOPT"] =~ /\s+-I\s*#{Regexp.escape isolate_lib}\b/ ENV["RUBYOPT"] = "#{ENV['RUBYOPT']} -I#{isolate_lib}" end ENV["GEM_PATH"] = path end bin = File.join path, "bin" unless ENV["PATH"].split(File::PATH_SEPARATOR).include? bin ENV["PATH"] = [bin, ENV["PATH"]].join File::PATH_SEPARATOR end ENV["ISOLATED"] = path if system? then Gem.path.unshift path # HACK: this is just wrong! Gem.path.uniq! # HACK: needed for the previous line :( end Isolate.refresh @enabled = true fire :enabled self end |
#enabled? ⇒ Boolean
183 184 185 |
# File 'lib/isolate/sandbox.rb', line 183 def enabled? @enabled end |
#environment(*environments, &block) ⇒ Object Also known as: env
Restricts gem calls inside block to a set of environments.
189 190 191 192 193 194 195 196 |
# File 'lib/isolate/sandbox.rb', line 189 def environment *environments, &block old = @environments @environments = @environments.dup.concat environments.map { |e| e.to_s } instance_eval(&block) ensure @environments = old end |
#gem(name, *requirements) ⇒ Object
Express a gem dependency. Works pretty much like RubyGems’ gem method, but respects environment and doesn’t activate ‘til later.
204 205 206 207 208 209 210 |
# File 'lib/isolate/sandbox.rb', line 204 def gem name, *requirements entry = entries.find { |e| e.name == name } return entry.update(*requirements) if entry entries << entry = Entry.new(self, name, *requirements) entry end |
#index ⇒ Object
A source index representing only isolated gems.
214 215 216 |
# File 'lib/isolate/sandbox.rb', line 214 def index @index ||= Gem::SourceIndex.from_gems_in File.join(path, "specifications") end |
#install(environment) ⇒ Object
:nodoc:
218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 |
# File 'lib/isolate/sandbox.rb', line 218 def install environment # :nodoc: fire :installing installable = entries.select do |e| !e.specification && e.matches?(environment) end unless installable.empty? padding = Math.log10(installable.size).to_i + 1 format = "[%0#{padding}d/%s] Isolating %s (%s)." installable.each_with_index do |entry, i| log format % [i + 1, installable.size, entry.name, entry.requirement] entry.install end Gem::Specification.reset end fire :installed self end |
#install? ⇒ Boolean
:nodoc:
242 243 244 |
# File 'lib/isolate/sandbox.rb', line 242 def install? # :nodoc: @options.fetch :install, true end |
#load(file) ⇒ Object
:nodoc:
246 247 248 249 |
# File 'lib/isolate/sandbox.rb', line 246 def load file # :nodoc: files << file instance_eval IO.read(file), file, 1 end |
#log(s) ⇒ Object
:nodoc:
251 252 253 |
# File 'lib/isolate/sandbox.rb', line 251 def log s # :nodoc: $stderr.puts s if verbose? end |
#multiruby? ⇒ Boolean
255 256 257 |
# File 'lib/isolate/sandbox.rb', line 255 def multiruby? @options.fetch :multiruby, true end |
#options(options = nil) ⇒ Object
259 260 261 262 |
# File 'lib/isolate/sandbox.rb', line 259 def = nil @options.merge! if @options end |
#path ⇒ Object
264 265 266 267 268 269 270 271 272 273 |
# File 'lib/isolate/sandbox.rb', line 264 def path base = @options.fetch :path, DEFAULT_PATH unless @options.key?(:multiruby) && @options[:multiruby] == false suffix = "#{Gem.ruby_engine}-#{RbConfig::CONFIG['ruby_version']}" base = File.join(base, suffix) unless base =~ /#{suffix}/ end File. base end |
#remove(*extra) ⇒ Object
275 276 277 278 279 280 281 282 283 284 285 286 287 288 289 290 291 292 293 294 |
# File 'lib/isolate/sandbox.rb', line 275 def remove(*extra) unless extra.empty? padding = Math.log10(extra.size).to_i + 1 format = "[%0#{padding}d/%s] Nuking %s." extra.each_with_index do |e, i| log format % [i + 1, extra.size, e.full_name] Gem::DefaultUserInteraction.use_ui Gem::SilentUI.new do uninstaller = Gem::Uninstaller.new(e.name, :version => e.version, :ignore => true, :executables => true, :install_dir => e.base_dir) uninstaller.uninstall end end end end |
#system? ⇒ Boolean
296 297 298 |
# File 'lib/isolate/sandbox.rb', line 296 def system? @options.fetch :system, true end |
#verbose? ⇒ Boolean
300 301 302 |
# File 'lib/isolate/sandbox.rb', line 300 def verbose? @options.fetch :verbose, true end |